Package: stlplus
Type: Package
Title: Enhanced Seasonal Decomposition of Time Series by Loess
Version: 0.5.2
Date: 2026-04-06
Authors@R: person("Ryan", "Hafen", email = "rhafen@gmail.com", role = c("aut", "cre"),
    comment = c(ORCID = "0000-0002-5516-8367"))
Maintainer: Ryan Hafen <rhafen@gmail.com>
Description: Decompose a time series into seasonal, trend, and remainder
    components using an implementation of Seasonal Decomposition of Time
    Series by Loess (STL) that provides several enhancements over the STL
    method in the stats package.  These enhancements include handling missing
    values, providing higher order (quadratic) loess smoothing with automated
    parameter choices, frequency component smoothing beyond the seasonal and
    trend components, and some basic plot methods for diagnostics.
LazyLoad: yes
NeedsCompilation: yes
Imports: lattice, yaImpute, stats, Rcpp
Suggests: testthat
LinkingTo: Rcpp
License: BSD_3_clause + file LICENSE
URL: https://github.com/hafen/stlplus
BugReports: https://github.com/hafen/stlplus/issues
Note: This is experimental software distributed free of charge and
        comes with no warranty!  Exercise professional caution.
RoxygenNote: 7.3.3
Packaged: 2026-04-14 03:56:01 UTC; rhafen
Author: Ryan Hafen [aut, cre] (ORCID: <https://orcid.org/0000-0002-5516-8367>)
Repository: CRAN
Date/Publication: 2026-04-14 04:20:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-04-15 04:58:12 UTC; unix
Archs: stlplus.so.dSYM

Package: stlplus
Type: Package
Title: Enhanced Seasonal Decomposition of Time Series by Loess
Version: 0.5.2
Date: 2026-04-06
Authors@R: person("Ryan", "Hafen", email = "rhafen@gmail.com", role = c("aut", "cre"),
    comment = c(ORCID = "0000-0002-5516-8367"))
Maintainer: Ryan Hafen <rhafen@gmail.com>
Description: Decompose a time series into seasonal, trend, and remainder
    components using an implementation of Seasonal Decomposition of Time
    Series by Loess (STL) that provides several enhancements over the STL
    method in the stats package.  These enhancements include handling missing
    values, providing higher order (quadratic) loess smoothing with automated
    parameter choices, frequency component smoothing beyond the seasonal and
    trend components, and some basic plot methods for diagnostics.
LazyLoad: yes
NeedsCompilation: yes
Imports: lattice, yaImpute, stats, Rcpp
Suggests: testthat
LinkingTo: Rcpp
License: BSD_3_clause + file LICENSE
URL: https://github.com/hafen/stlplus
BugReports: https://github.com/hafen/stlplus/issues
Note: This is experimental software distributed free of charge and
        comes with no warranty!  Exercise professional caution.
RoxygenNote: 7.3.3
Packaged: 2026-04-14 03:56:01 UTC; rhafen
Author: Ryan Hafen [aut, cre] (ORCID: <https://orcid.org/0000-0002-5516-8367>)
Repository: CRAN
Date/Publication: 2026-04-14 04:20:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-27 06:55:46 UTC; unix
Archs: stlplus.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(fitted,stl)
S3method(fitted,stlplus)
S3method(plot,stlplus)
S3method(predict,stl)
S3method(predict,stlplus)
S3method(remainder,stl)
S3method(remainder,stlplus)
S3method(seasonal,stl)
S3method(seasonal,stlplus)
S3method(stlplus,default)
S3method(stlplus,ts)
S3method(stlplus,zoo)
S3method(time,stl)
S3method(time,stlplus)
S3method(trend,stl)
S3method(trend,stlplus)
export(fc)
export(getraw)
export(plot_cycle)
export(plot_rembycycle)
export(plot_seasonal)
export(plot_trend)
export(remainder)
export(seasonal)
export(stlplus)
export(trend)
importFrom(Rcpp,evalCpp)
importFrom(lattice,panel.abline)
importFrom(lattice,panel.loess)
importFrom(lattice,panel.segments)
importFrom(lattice,panel.xyplot)
importFrom(lattice,xyplot)
importFrom(stats,frequency)
importFrom(stats,loess)
importFrom(stats,median)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,time)
importFrom(stats,weighted.mean)
importFrom(utils,head)
importFrom(utils,stack)
importFrom(utils,tail)
importFrom(yaImpute,ann)
useDynLib(stlplus, .registration = TRUE)

## stlplus

[![R-CMD-check](https://github.com/hafen/stlplus/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/hafen/stlplus/actions/workflows/R-CMD-check.yaml)
[![CRAN](http://www.r-pkg.org/badges/version/stlplus)](https://CRAN.R-project.org/package=stlplus)
![](http://cranlogs.r-pkg.org/badges/stlplus)

![png](https://cloud.githubusercontent.com/assets/1275592/11673681/b7fce548-9dcf-11e5-8cd2-f3311b501ab9.png)

This package contains enhancements to the Seasonal Trend Decomposition using Loess (STL) implementation that comes with base R, `stl()`.

Here are some of the added features over `stl()`:

- Can handle NA values
- Higher order loess smoothing (more than just local constant and linear)
- Automated parameter choices for local quadratic
- Frequency component smoothing beyond seasonal and trend
- Plot methods for diagnostics

For (very) experimental inference, prediction, and variance reduction at endpoints, see the [operator](https://github.com/hafen/operator) package.

## References

- Cleveland, R. B., Cleveland, W. S., McRae, J. E., & Terpenning, I. (1990). STL: A seasonal-trend decomposition procedure based on loess. *Journal of Official Statistics*, 6(1), 3-73.
- Hafen, R. P. "Local regression models: Advancements, applications, and new methods." (2010). <https://docs.lib.purdue.edu/dissertations/AAI3413793/>

## Installation

CRAN:

```s
install.packages("stlplus")
```

Github:

```s
devtools::install_github("hafen/stlplus")
```

## License

This software is released under the BSD license.  Please read the [license](https://github.com/hafen/stlplus/blob/master/LICENSE.md) document.

#' Lattice plot of the raw, seasonal, trend, and remainder components
#'
#' Lattice plot of the raw, seasonal, trend, and remainder components.  If post-trend smoothing was done, these components will be plotted instead of the trend component.
#'
#' @param x object of class \code{"stlplus"}.
#' @param scales,type,as.table,strip,strip.left,between,layout,\ldots parameters to be passed to xyplot.
#' @return object of class \code{"trellis"}.
#'
#' @references R. B. Cleveland, W. S. Cleveland, J. E.  McRae, and I. Terpenning (1990) STL:  A  Seasonal-Trend  Decomposition  Procedure Based on Loess. \emph{Journal of Official Statistics}, \bold{6}, 3--73.
#' @seealso \code{\link{stlplus}}
#' @export
#' @importFrom lattice xyplot panel.abline panel.xyplot panel.loess panel.segments
plot.stlplus <- function(
  x,
  scales = list(y = list(relation = "sliced")),
  type = "l",
  as.table = TRUE,
  strip = FALSE,
  strip.left = TRUE,
  between = list(y = 0.5),
  layout = NULL,
  ...
) {
  if (x$pars$fc.number == 0) {
    d <- data.frame(
      time = rep(time.stlplus(x), 4),
      values = c(getraw(x), seasonal(x), trend(x), remainder(x)),
      ind = factor(rep(c(1:4), each = x$n))
    )
    levels(d$ind) <- c("raw", "seasonal", "trend", "remainder")
    d$which <- "isnotNA"

    if (is.null(layout)) {
      layout <- c(1, 4)
    }

    # if there are any NA values, plot the part of the seasonal and trend
    # components with a different color
    if (any(is.na(getraw(x)))) {
      d$values[d$ind == "seasonal" & is.na(getraw(x))] <- NA
      d$values[d$ind == "trend" & is.na(getraw(x))] <- NA
      d <- rbind(
        d,
        data.frame(
          time = rep(time.stlplus(x), 2),
          values = c(seasonal(x), trend(x)),
          ind = rep(c("seasonal", "trend"), each = x$n),
          which = "isxNA"
        )
      )
      d$values[
        d$ind == "seasonal" & d$which == "isxNA" & !is.na(getraw(x))
      ] <- NA
      d$values[d$ind == "trend" & d$which == "isxNA" & !is.na(getraw(x))] <- NA
    }
  } else {
    fc.number <- x$pars$fc.number
    nvar <- 3 + fc.number
    fc.name <- as.character(x$pars$fc$fc.name)
    if (fc.number == 1) {
      fcdat <- x$fc[, 1]
    } else {
      fcdat <- stack(x$fc[, 1:fc.number])$values
    }

    d <- data.frame(
      time = rep(time.stlplus(x), nvar),
      values = c(getraw(x), seasonal(x), fcdat, remainder(x)),
      ind = factor(rep(c(1:nvar), each = x$n))
    )
    levels(d$ind) <- c("raw", "seasonal", fc.name, "remainder")
    d$which <- "isnotNA"

    if (is.null(layout)) {
      layout <- c(1, nvar)
    }

    # if there are any NA values, plot the part of the seasonal and trend
    # components with a different color
    if (any(is.na(getraw(x)))) {
      d$values[d$ind == "seasonal" & is.na(getraw(x))] <- NA
      for (i in 1:fc.number) {
        d$values[d$ind == fc.name[i] & is.na(getraw(x))] <- NA
      }
      d <- rbind(
        d,
        data.frame(
          time = rep(time.stlplus(x), fc.number + 1),
          values = c(seasonal(x), fcdat),
          ind = rep(c("seasonal", fc.name), each = x$n),
          which = "isxNA"
        )
      )
      d$values[
        d$ind == "seasonal" & d$which == "isxNA" & !is.na(getraw(x))
      ] <- NA
      for (i in 1:fc.number) {
        d$values[
          d$ind == fc.name[i] & d$which == "isxNA" & !is.na(getraw(x))
        ] <- NA
      }
    }
  }

  p <- lattice::xyplot(
    values ~ time | ind,
    data = d,
    groups = which,
    type = type,
    layout = layout,
    scales = scales,
    as.table = as.table,
    strip = strip,
    strip.left = strip.left,
    between = between,
    ...
  )
  p
}

.midmean <- function(x) {
  q <- quantile(x, probs = c(0.25, 0.75), na.rm = TRUE)
  isInner <- x < q[2] & x > q[1]
  mean(x[isInner], na.rm = TRUE)
}

#' Cycle-Subseries Plot for an stlplus Object
#'
#' Plots the seasonal component by cycle-subseries, with lines emanating
#' from the midmean of the values within each cycle-subseries.
#'
#' @param x object of class \code{"stlplus"}.
#' @param layout,col,xlab,ylab,panel,\ldots parameters to be passed to \code{xyplot()}.
#' @return object of class \code{"trellis"}.
#' @references R. B. Cleveland, W. S. Cleveland, J. E.  McRae, and I. Terpenning (1990) STL:  A  Seasonal-Trend  Decomposition  Procedure Based on Loess. \emph{Journal of Official Statistics}, \bold{6}, 3--73.
#' @seealso \code{\link{stlplus}}
#' @export
plot_cycle <- function(
  x,
  layout = c(x$pars$n.p, 1),
  col = "#0080ff",
  xlab = "Time",
  ylab = "Seasonal",
  panel = function(x, y, ...) {
    lattice::panel.segments(x, rep(.midmean(y), length(x)), x, y, col = col)
  },
  ...
) {
  seas <- seasonal(x)
  t <- time.stlplus(x)

  cycle_sub_idx <- x$data$sub.labels

  if (x$pars$periodic) {
    p <- lattice::xyplot(
      seas ~ t | cycle_sub_idx,
      layout = layout,
      type = "l",
      xlab = xlab,
      ylab = ylab,
      ...
    )
  } else {
    p <- lattice::xyplot(
      seas ~ t | cycle_sub_idx,
      layout = layout,
      type = "l",
      panel = panel,
      xlab = xlab,
      ylab = ylab,
      ...
    )
  }

  p
}

#' Seasonal Diagnostic Plot for an stlplus Object
#'
#' Plots each cycle-subseries of the detrended data (or equivalently,
#' seasonal plus remainder), centered by the mean seasonal component for each
#' subseries. The fitted seasonal component is plotted as a line through the
#' points.
#'
#' @param x object of class \code{"stlplus"}.
#' @param col,lwd,xlab,ylab,\ldots parameters to be passed to \code{xyplot()}.
#'
#' @details Helps decide how much of the variation in the data other than the trend should go into the seasonal component, and how much in the remainder.
#' @return object of class \code{"trellis"}.
#' @references R. B. Cleveland, W. S. Cleveland, J. E.  McRae, and I. Terpenning (1990) STL:  A  Seasonal-Trend  Decomposition  Procedure Based on Loess. \emph{Journal of Official Statistics}, \bold{6}, 3--73.
#' @seealso \code{\link{stlplus}}
#' @export
plot_seasonal <- function(
  x,
  col = c("darkgray", "black"),
  lwd = 2,
  xlab = "Time",
  ylab = "Seasonal + Remainder\n(centered by seasonal mean)",
  ...
) {
  dat <- by(
    data.frame(
      v1 = seasonal(x) + remainder(x),
      v2 = seasonal(x),
      t = time.stlplus(x),
      v3 = x$data$sub.labels
    ),
    list(x$data$sub.labels),
    function(dd) {
      mn <- mean(dd$v2, na.rm = TRUE)
      data.frame(a = dd$v1 - mn, b = dd$v2 - mn, t = dd$t, sub.labels = dd$v3)
    }
  )

  dat <- do.call(rbind, dat)

  p <- lattice::xyplot(
    a + b ~ t | sub.labels,
    data = dat,
    type = c("p", "l"),
    col = col,
    lwd = lwd,
    distribute.type = TRUE,
    as.table = TRUE,
    xlab = xlab,
    ylab = ylab,
    ...
  )
  p
}

#' Plot of Remainder Component by Cycle-Subseries
#'
#' Plots the remainder component by cycle-subseries with a loess line.
#'
#' @param x object of class \code{"stlplus"}.
#' @param col,locol,lolwd,xlab,ylab,\ldots parameters to be passed to \code{xyplot()}. \code{locol} and \code{lolwd} are the line color and width for the loess line.
#'
#' @return object of class \code{"trellis"}.
#' @references R. B. Cleveland, W. S. Cleveland, J. E.  McRae, and I. Terpenning (1990) STL:  A  Seasonal-Trend  Decomposition  Procedure Based on Loess. \emph{Journal of Official Statistics}, \bold{6}, 3--73.
#' @seealso \code{\link{stlplus}}
#' @export
plot_rembycycle <- function(
  x,
  col = "darkgray",
  locol = "black",
  lolwd = 2,
  xlab = "Time",
  ylab = "Remainder",
  ...
) {
  vals2 <- data.frame(
    values = remainder(x),
    ind = x$data$sub.labels,
    t = time.stlplus(x)
  )

  vals2 <- vals2[!is.na(vals2$values), ]

  p <- lattice::xyplot(
    values ~ t | ind,
    data = vals2,
    type = "p",
    panel = function(x, y, ...) {
      lattice::panel.abline(h = 0, lty = 2, col = "darkgray")
      lattice::panel.xyplot(x, y, ...)
      lattice::panel.loess(x, y, col = locol, lwd = lolwd)
    },
    xlab = xlab,
    ylab = ylab,
    col = col,
    as.table = TRUE,
    ...
  )

  p
}

#' Trend Diagnostic Plot for an stlplus Object
#'
#' Plots the trend+remainder with the trend component overlaid, and the
#' remainder component in a separate panel.
#'
#' @param x object of class \code{"stlplus"}.
#' @param xlab,ylab,span,type,scales,lwd,col,layout parameters to be passed to
#'   xyplot.
#' @param between,strip,strip.left,as.table,\ldots parameters to be passed to
#' xyplot.
#'
#' @return object of class \code{"trellis"}.
#' @references R. B. Cleveland, W. S. Cleveland, J. E.  McRae, and I. Terpenning (1990) STL:  A  Seasonal-Trend  Decomposition  Procedure Based on Loess. \emph{Journal of Official Statistics}, \bold{6}, 3--73.
#' @seealso \code{\link{stlplus}}
#' @export
plot_trend <- function(
  x,
  xlab = "Time",
  ylab = "Trend",
  span = 0.3,
  type = c("p", "l"),
  scales = list(y = list(relation = "free")),
  lwd = c(1, 1),
  col = c("darkgray", "black", "darkgray"),
  layout = c(1, 2),
  between = list(y = 0.5),
  strip = FALSE,
  strip.left = TRUE,
  as.table = TRUE,
  ...
) {
  dat <- rbind(
    data.frame(
      x = time.stlplus(x),
      y = remainder(x) + trend(x),
      type = "p",
      pan = "Trend"
    ),
    data.frame(
      x = time.stlplus(x),
      y = trend(x),
      type = "l",
      pan = "Trend"
    ),
    data.frame(
      x = time.stlplus(x),
      y = remainder(x),
      type = "p",
      pan = "Remainder"
    ),
    data.frame(
      x = time.stlplus(x),
      y = predict(
        loess(
          remainder(x) ~ c(seq_along(remainder(x))),
          span = span,
          weights = x$data$weights
        ),
        newdata = c(seq_along(remainder(x)))
      ),
      type = "l",
      pan = "Remainder"
    )
  )

  p <- lattice::xyplot(
    y ~ x | pan,
    groups = type,
    data = dat,
    type = type,
    col = col,
    as.table = as.table,
    layout = layout,
    lwd = lwd,
    scales = scales,
    distribute.type = TRUE,
    between = between,
    strip = strip,
    strip.left = strip.left,
    xlab = xlab,
    ylab = ylab,
    ...
  )

  p
}

#' Accessor functions for elements of an stl and stlplus object
#'
#'Retrieves the raw, seasonal, trend, remainder, or time components from an stlplus object.  The methods \code{seasonal.stl}, ... also exist as a convenience for extracting components from R's \code{stl()}.
#'
#' @param x,object object of class \code{"stl"} or \code{"stlplus"}.
#' @param fcnum number of post-trend smoothing frequency component.
#' @param \ldots additional parameters
#'
#' @return
#' Returns a vector of either the \code{getraw} time series, the \code{seasonal}, \code{trend}, or \code{remainder} components, or the \code{time} values of the time series.  If \code{time}s are requested but were not supplied in the initial \code{stlplus} call, the \code{1:n} vector is returned, where \code{n} is the number of data points.  The \code{fitted} method returns the sum of the seasonal and trend.
#' @references R. B. Cleveland, W. S. Cleveland, J.E.  McRae, and I. Terpenning (1990) STL:  A  Seasonal-Trend  Decomposition  Procedure Based on Loess. \emph{Journal of Official Statistics}, \bold{6}, 3--73.
#' @note
#' The \code{fitted} and \code{predict} methods are equivalent.  For objects of class \code{"stlplus"}, these functions return the sum of all components but the remainder, including post-trend smoothing components.  Note also that the \code{trend} method for objects of class \code{"stlplus"} only returns the trend component from the STL iterations, even when post-trend smoothing is done.
#' @seealso \code{\link{stlplus}}
#' @examples
#' co2.stl <- stlplus(co2, t = as.vector(stats::time(co2)), n.p=12, l.window=13,
#' t.window=19, s.window=35, s.degree=1, sub.labels = substr(month.name, 1, 3))
#'
#' plot(seasonal(co2.stl))
#' @export
#' @rdname accessors
seasonal <- function(object) UseMethod("seasonal")

#' @export
#' @rdname accessors
trend <- function(object) UseMethod("trend")

#' @export
#' @rdname accessors
remainder <- function(object) UseMethod("remainder")

#' @export
#' @rdname accessors
getraw <- function(object) {
  object$data$raw
}

#' @export
#' @rdname accessors
remainder.stlplus <- function(object) {
  if (object$pars$fc.number == 0) {
    object$data$remainder
  } else {
    object$fc$remainder
  }
}

#' @export
#' @rdname accessors
fitted.stlplus <- function(object, ...) {
  if (object$pars$fc.number == 0) {
    object$data$seasonal + object$data$trend
  } else {
    object$data$seasonal + apply(object$fc[, 1:object$pars$fc.number], 1, sum)
  }
}

#' @export
#' @rdname accessors
predict.stlplus <- function(object, ...) {
  if (object$pars$fc.number == 0) {
    object$data$seasonal + object$data$trend
  } else {
    object$data$seasonal + apply(object$fc[, 1:object$pars$fc.number], 1, sum)
  }
}

#' @export
#' @rdname accessors
seasonal.stlplus <- function(object) {
  object$data$seasonal
}

#' @export
#' @rdname accessors
trend.stlplus <- function(object) {
  object$data$trend
}

# get frequency components:
#' @export
#' @rdname accessors
fc <- function(object, fcnum = 1) {
  if (!"stlplus" %in% class(object)) {
    stop("object not of class stlplus")
  }

  if (is.null(object$fc)) {
    stop("there are no post-trend frequency components")
  }

  if (fcnum > ncol(object$fc)) {
    stop("there are not that many frequency components")
  }

  object$fc[, fcnum]
}

# setGeneric("time")
#' @export
#' @rdname accessors
time.stlplus <- function(x, ...) {
  if (length(x$t) == x$n) {
    x$t
  } else {
    c(1:x$n)
  }
}

# now for objects of class stl

#' @export
#' @rdname accessors
remainder.stl <- function(object) {
  as.numeric(object$time.series[, 3])
}

#' @export
#' @rdname accessors
seasonal.stl <- function(object) {
  as.numeric(object$time.series[, 1])
}

#' @export
#' @rdname accessors
trend.stl <- function(object) {
  as.numeric(object$time.series[, 2])
}

# setGeneric("time")
#' @export
#' @rdname accessors
time.stl <- function(x, ...) {
  as.numeric(stats::time(x$time.series))
}

#' @export
#' @rdname accessors
predict.stl <- function(object, ...) {
  seasonal(object) + trend(object)
}

#' @export
#' @rdname accessors
fitted.stl <- function(object, ...) {
  seasonal(object) + trend(object)
}

test_that("direct loess matches R's loess", {
  n <- 100
  x <- 1:n
  set.seed(34565)
  y <- rnorm(n)

  dc <- loess.control(surface = "direct")

  # we can only test indices 16:85 because for some reason
  # on Windows i386 only, the non-symmetric fits are off

  r1 <- stlplus:::.loess_stlplus(x = x, y = y, span = 31, degree = 0)
  r2 <- predict(loess(y ~ x, degree = 0, span = 31 / n, control = dc))
  expect_true(mean(abs(r1 - r2)[16:85]) < 1.0e-15)

  r1 <- stlplus:::.loess_stlplus(x = x, y = y, span = 31, degree = 1)
  r2 <- predict(loess(y ~ x, degree = 1, span = 31 / n, control = dc))
  expect_true(mean(abs(r1 - r2)[16:85]) < 1.0e-15)

  r1 <- stlplus:::.loess_stlplus(x = x, y = y, span = 31, degree = 2)
  r2 <- predict(loess(y ~ x, degree = 2, span = 31 / n, control = dc))
  expect_true(mean(abs(r1 - r2)[16:85]) < 1.0e-15)
})

test_that("stlplus matches stl", {
  x1 <- stlplus(
    co2,
    t = as.vector(time(co2)),
    n.p = 12,
    l.window = 13,
    t.window = 19,
    s.window = 11,
    s.degree = 1,
    outer = 1,
    s.jump = 1,
    t.jump = 1,
    l.jump = 1
  )
  x2 <- stl(
    co2,
    l.window = 13,
    t.window = 19,
    s.window = 11,
    s.degree = 1,
    s.jump = 1,
    t.jump = 1,
    l.jump = 1,
    robust = FALSE
  )
  expect_true(mean(abs(seasonal(x1) - seasonal(x2))) < 1.0e-10)
  expect_true(mean(abs(trend(x1) - trend(x2))) < 1.0e-10)

  x1 <- stlplus(
    co2,
    t = as.vector(time(co2)),
    n.p = 12,
    l.window = 13,
    t.window = 39,
    s.window = 101,
    s.degree = 1,
    outer = 1,
    s.jump = 1,
    t.jump = 1,
    l.jump = 1
  )
  x2 <- stl(
    co2,
    l.window = 13,
    t.window = 39,
    s.window = 101,
    s.degree = 1,
    s.jump = 1,
    t.jump = 1,
    l.jump = 1,
    robust = FALSE
  )
  expect_true(mean(abs(seasonal(x1) - seasonal(x2))) < 1.0e-10)
  expect_true(mean(abs(trend(x1) - trend(x2))) < 1.0e-10)
})

test_that("periodic stlplus matches stl", {
  x1 <- stlplus(
    co2,
    t = as.vector(time(co2)),
    n.p = 12,
    l.window = 13,
    t.window = 19,
    s.window = "periodic",
    outer = 1,
    s.jump = 1,
    t.jump = 1,
    l.jump = 1
  )
  x2 <- stl(
    co2,
    l.window = 13,
    t.window = 19,
    s.window = "periodic",
    s.jump = 1,
    t.jump = 1,
    l.jump = 1,
    robust = FALSE
  )
  expect_true(mean(abs(seasonal(x1) - seasonal(x2))) < 1.0e-6)
  expect_true(mean(abs(trend(x1) - trend(x2))) < 1.0e-6)
})

# compare to loess with missing values
# y[10] <- NA
# r1 <- .loess_stlplus(x = x, y = y, span = 31, degree = 1)
# r2 <- predict(loess(y ~ x, degree = 1, span = 31/n, control = dc), newdata = c(1:n))
# plot(r1-r2)
# stopifnot(mean(abs(r1-r2)) < 1.0e-15)

## doesn't match with loess when span > n (but matches stl())
##---------------------------------------------------------

# r1 <- .loess_stlplus(x = x, y = y, span = 131, degree = 0)
# r2 <- predict(loess(y ~ x, degree = 0, span = 131 / n, control = dc))
# stopifnot(mean(abs(r1-r2)) < 1.0e-15)
#
# r1 <- .loess_stlplus(x = x, y = y, span = 131, degree = 1)
# r2 <- predict(loess(y ~ x, degree = 1, span = 131 / n, control = dc))
# stopifnot(mean(abs(r1-r2)) < 1.0e-15)
#
# r1 <- .loess_stlplus(x = x, y = y, span = 131, degree = 2)
# r2 <- predict(loess(y ~ x, degree = 2, span = 131 / n, control = dc))
# stopifnot(mean(abs(r1-r2)) < 1.0e-15)

## compare to operator
##---------------------------------------------------------

# library(operator)
# n <- 100
# set.seed(5432)
# y <- rnorm(100)
# x1 <- stlplus(y, n.p = 12, l.window = 13, t.window = 19, s.window = 11, s.degree = 1, s.jump = 1, t.jump = 1, l.jump = 1)
# x2 <- stlOp(x1)
# stopifnot(mean(abs(seasonal(x1) - x2$seas$O %*% y)) < 1.0e-12)
# stopifnot(mean(abs(trend(x1) - x2$trend$O %*% y))  < 1.0e-12)

#' Seasonal Decomposition of Time Series by Loess
#'
#' Decompose a time series into seasonal, trend and irregular components using \code{loess}, acronym STL.  A new implementation of STL.  Allows for NA values, local quadratic smoothing,  post-trend smoothing, and endpoint blending.  The usage is very similar to that of R's built-in \code{stl()}.
#'
#' @param x vector of time series values, in order of time.  If \code{x} is a time series object, then \code{t} and \code{n.p} do not need to be specified, although they still can be.
#' @param t times at which the time series values were observed.  Not required.
#' @param n.p periodicity of the seasonal component.  In R's \code{stl} function, this is the frequency of the time series.
#' @param s.window either the character string \code{"periodic"} or the span (in lags) of the loess window for seasonal extraction, which should be odd.  This has no default.
#' @param s.degree degree of locally-fitted polynomial in seasonal extraction.  Should be 0, 1, or 2.
#' @param t.window the span (in lags) of the loess window for trend extraction, which should be odd.  If \code{NULL}, the default, \code{nextodd(ceiling((1.5*period) / (1-(1.5/s.window))))}, is taken.
#' @param t.degree degree of locally-fitted polynomial in trend extraction.  Should be 0, 1, or 2.
#' @param l.window the span (in lags) of the loess window of the low-pass filter used for each subseries.  Defaults to the smallest odd integer greater than or equal to \code{n.p} which is recommended since it prevents competition between the trend and seasonal components.  If not an odd integer its given value is increased to the next odd one.
#' @param l.degree degree of locally-fitted polynomial for the subseries low-pass filter.  Should be 0, 1, or 2.
#' @param critfreq the critical frequency to use for automatic calculation of smoothing windows for the trend and high-pass filter.
#' @param fc.window vector of lengths of windows for loess smoothings for other trend frequency components after the original STL decomposition has been obtained.  The smoothing is applied to the data with the STL seasonal component removed.  A frequency component is computed by a loess fit with the window length equal to the first element of fc.window, the component is removed, another component is computed with the window length equal to the second element of fc.window, and so forth. In most cases, the values of the argument should be decreasing, that is, the frequency bands of the fitted components should increase. The robustness weights from original STL are used as weights in the loess fitting if specified.
#' @param fc.degree vector of degrees of locally-fitted polynomial in the loess smoothings for the frequency components specified in fc.window. Values of 0, 1 and 2 are allowed. If the length of fc.degree is less than that of fc.window, the former is expanded to the length of the latter using rep; thus, giving the value 1 specifies a degree of 1 for all components.
#' @param fc.name vector of names of the post-trend smoothing operations specified by \code{fc.window} and \code{fc.degree} (optional).
#' @param inner integer; the number of \sQuote{inner} (backfitting) iterations; usually very few (2) iterations suffice.
#' @param outer integer; the number of \sQuote{outer} robustness iterations.  Default is 0, but Recommended if outliers are present.
#' @param sub.labels optional vector of length n.p that contains the labels of the subseries in their natural order (such as month name, day of week, etc.), used for strip labels when plotting.  All entries must be unique.
#' @param sub.start which element of sub.labels does the series begin with.  See details.
#' @param zero.weight value to use as zero for zero weighting
#' @param details if \code{TRUE}, returns a list of the results of all the intermediate iterations.
#' @param s.jump,t.jump,l.jump,fc.jump integers at least one to increase speed of the respective smoother.  Linear interpolation happens between every \code{*.jump}th value.
#' @param s.blend,t.blend,l.blend,fc.blend vectors of proportion of blending to degree 0 polynomials at the endpoints of the series.
#' @param \ldots additional parameters
#' @details The seasonal component is found by \emph{loess} smoothing the seasonal sub-series (the series of all January values, \ldots); if \code{s.window = "periodic"} smoothing is effectively replaced by taking the mean. The seasonal values are removed, and the remainder smoothed to find the trend. The overall level is removed from the seasonal component and added to the trend component. This process is iterated a few times.  The \code{remainder} component is the residuals from the seasonal plus trend fit.
#'
#' Cycle-subseries labels are useful for plotting and can be specified through the sub.labels argument.  Here is an example for how the sub.labels and sub.start parameters might be set for one situation.  Suppose we have a daily series with n.p=7 (fitting a day-of-week component).  Here, sub.labels could be set to c("Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat").  Now, if the series starts with a Wednesday value, then one would specify sub.labels=4, since Wednesday is the fourth element of sub.labels.  This ensures that the labels in the plots to start the plotting with Sunday cycle-subseries instead of Wednesday.
#' @return
#' returns an object of class \code{"stlplus"}, containing
#' \item{data}{data frame containing all of the components: \code{raw}, \code{seasonal}, \code{trend}, \code{remainder}, \code{weights}.}
#' \item{pars}{list of parameters used in the procedure.}
#' \item{fc.number}{number of post-trend frequency components fitted.}
#' \item{fc}{data frame of the post-trend frequency components.}
#' \item{time}{vector of time values corresponding to the raw values, if specified.}
#' \item{n}{the number of observations.}
#' \item{sub.labels}{the cycle-subseries labels.}
#' @references R. B. Cleveland, W. S. Cleveland, J. E.  McRae, and I. Terpenning (1990) STL:  A  Seasonal-Trend  Decomposition  Procedure Based on Loess. \emph{Journal of Official Statistics}, \bold{6}, 3--73.
#' @author Ryan Hafen
#' @note This is a complete re-implementation of the STL algorithm, with the loess part in C and the rest in R.  Moving a lot of the code to R makes it easier to experiment with the method at a very minimal speed cost.  Recoding in C instead of using R's built-in loess results in better performance, especially for larger series.
#' @seealso \code{\link{plot.stlplus}} for plotting the components, \code{\link{getraw}}, \code{\link{seasonal}}, \code{\link{trend}}, \code{\link{remainder}} for accessing the components.
#' @example man-roxygen/ex-stlplus.R
#' @importFrom Rcpp evalCpp
#' @importFrom stats frequency loess median predict quantile weighted.mean time
#' @importFrom utils head stack tail
#' @export
#' @rdname stlplus
stlplus <- function(
  x,
  t = NULL,
  n.p,
  s.window,
  s.degree = 1,
  t.window = NULL,
  t.degree = 1,
  fc.window = NULL,
  fc.degree = NULL,
  fc.name = NULL,
  l.window = NULL,
  l.degree = t.degree,
  s.jump = ceiling(s.window / 10),
  t.jump = ceiling(t.window / 10),
  l.jump = ceiling(l.window / 10),
  fc.jump = NULL,
  critfreq = 0.05,
  s.blend = 0,
  t.blend = 0,
  l.blend = t.blend,
  fc.blend = NULL,
  inner = 2,
  outer = 1,
  sub.labels = NULL,
  sub.start = 1,
  zero.weight = 1e-6,
  details = FALSE,
  ...
) {
  UseMethod("stlplus")
}

#' @export
#' @rdname stlplus
stlplus.ts <- function(
  x,
  t = as.numeric(stats::time(x)),
  n.p = frequency(x),
  s.window,
  s.degree = 1,
  t.window = NULL,
  t.degree = 1,
  fc.window = NULL,
  fc.degree = NULL,
  fc.name = NULL,
  l.window = NULL,
  l.degree = t.degree,
  s.jump = ceiling(s.window / 10),
  t.jump = ceiling(t.window / 10),
  l.jump = ceiling(l.window / 10),
  fc.jump = NULL,
  critfreq = 0.05,
  s.blend = 0,
  t.blend = 0,
  l.blend = t.blend,
  fc.blend = NULL,
  inner = 2,
  outer = 1,
  sub.labels = NULL,
  sub.start = 1,
  zero.weight = 1e-6,
  details = FALSE,
  ...
) {
  if (is.matrix(x)) {
    stop("only univariate series are allowed")
  }

  if (missing(n.p)) {
    n.p <- frequency(x)
  }
  if (!is.null(t)) {
    if (length(t) != length(x)) {
      stop("t must be same length as time series")
    }
  } else {
    t <- as.vector(stats::time(x))
  }

  stlplus.default(
    x,
    t = t,
    n.p = n.p,
    s.window = s.window,
    s.degree = s.degree,
    t.window = t.window,
    t.degree = t.degree,
    fc.window = fc.window,
    fc.degree = fc.degree,
    fc.name = fc.name,
    l.window = l.window,
    l.degree = l.degree,
    s.jump = s.jump,
    t.jump = t.jump,
    l.jump = l.jump,
    fc.jump = fc.jump,
    critfreq = 0.05,
    s.blend = s.blend,
    t.blend = t.blend,
    l.blend = l.blend,
    fc.blend = NULL,
    inner = inner,
    outer = outer,
    sub.labels = sub.labels,
    sub.start = sub.start,
    details = details,
    ...
  )
}

#' @export
#' @rdname stlplus
stlplus.zoo <- function(...) {
  stlplus.ts(...)
}

#' @export
stlplus.default <- function(
  x,
  t = NULL,
  n.p,
  s.window,
  s.degree = 1,
  t.window = NULL,
  t.degree = 1,
  fc.window = NULL,
  fc.degree = NULL,
  fc.name = NULL,
  l.window = NULL,
  l.degree = t.degree,
  s.jump = ceiling(s.window / 10),
  t.jump = ceiling(t.window / 10),
  l.jump = ceiling(l.window / 10),
  fc.jump = NULL,
  critfreq = 0.05,
  s.blend = 0,
  t.blend = 0,
  l.blend = t.blend,
  fc.blend = NULL,
  inner = 2,
  outer = 1,
  sub.labels = NULL,
  sub.start = 1,
  zero.weight = 1e-6,
  details = FALSE,
  ...
) {
  if (missing(n.p)) {
    stop(
      "must specify periodicity of seasonal (either explicitly or through a time series object)"
    )
  }
  n.p <- as.integer(n.p)

  if (n.p < 4) {
    stop(paste(
      "Parameter n.p was set to ",
      n.p,
      ".  Must be at least 4.",
      sep = ""
    ))
  }

  # family <- ifelse(robust, "symmetric", "gaussian")

  Y <- as.vector(x)
  nan_idx <- which(is.nan(Y) | is.infinite(Y))
  if (length(nan_idx) > 0) {
    warning("NAs for infinite or NaN values introduced by coercion")
    Y[nan_idx] <- NA
  }
  n <- length(Y)
  nextodd <- function(x) {
    x <- round(x)
    x2 <- ifelse(x %% 2 == 0, x + 1, x)
    # if(any(x != x2))
    #   warning("A smoothing span was not odd, was rounded to nearest odd.  Check final object parameters to see which spans were used.")
    as.integer(x2)
  }

  wincheck <- function(x) {
    x <- nextodd(x)
    if (any(x <= 0)) {
      stop("Window lengths must be positive.")
    }
    x
  }

  degcheck <- function(x) {
    if (!all(x == 0 | x == 1 | x == 2)) {
      stop("Smoothing degree must be 0, 1, or 2")
    }
  }

  get.t.window <- function(t.dg, s.dg, n.s, n.p, omega) {
    if (t.dg == 0) {
      t.dg <- 1
    }
    if (s.dg == 0) {
      s.dg <- 1
    }

    coefs_a <- data.frame(
      a = c(0.000103350651767650, 3.81086166990428e-6),
      b = c(-0.000216653946625270, 0.000708495976681902)
    )
    coefs_b <- data.frame(
      a = c(1.42686036792937, 2.24089552678906),
      b = c(-3.1503819836694, -3.30435316073732),
      c = c(5.07481807116087, 5.08099438760489)
    )
    coefs_c <- data.frame(
      a = c(1.66534145060448, 2.33114333880815),
      b = c(-3.87719398039131, -1.8314816166323),
      c = c(6.46952900183769, 1.85431548427732)
    )

    # estimate critical frequency for seasonal
    betac0 <- coefs_a$a[s.dg] + coefs_a$b[s.dg] * omega
    betac1 <- coefs_b$a[s.dg] +
      coefs_b$b[s.dg] * omega +
      coefs_b$c[s.dg] * omega^2
    betac2 <- coefs_c$a[s.dg] +
      coefs_c$b[s.dg] * omega +
      coefs_c$c[s.dg] * omega^2
    f_c <- (1 - (betac0 + betac1 / n.s + betac2 / n.s^2)) / n.p

    # choose
    betat0 <- coefs_a$a[t.dg] + coefs_a$b[t.dg] * omega
    betat1 <- coefs_b$a[t.dg] +
      coefs_b$b[t.dg] * omega +
      coefs_b$c[t.dg] * omega^2
    betat2 <- coefs_c$a[t.dg] +
      coefs_c$b[t.dg] * omega +
      coefs_c$c[t.dg] * omega^2

    betat00 <- betat0 - f_c

    n.t <- nextodd(
      (-betat1 - sqrt(betat1^2 - 4 * betat00 * betat2)) / (2 * betat00)
    )

    n.t
  }

  y_idx <- !is.na(Y)
  noNA <- all(y_idx)

  csSmooth <- function(d) {
    nn <- length(d$y)

    # check to make sure there is at least one valid value
    if (all(is.na(d$y))) {
      a <- rep(NA, 1:(nn + 2))
    } else {
      cs.ev <- seq(1, length(d$y), by = s.jump)
      if (tail(cs.ev, 1) != nn) {
        cs.ev <- c(cs.ev, nn)
      }
      cs.ev <- c(0, cs.ev, nn + 1)
      # print(m)
      a <- .loess_stlplus(
        y = d$y,
        span = s.window,
        degree = s.degree,
        weights = d$w,
        m = cs.ev,
        noNA = noNA,
        jump = s.jump,
        at = c(1:(nn + 2))
      )
    }
    c(a, rep(NA, csLength - nn - 2))
  }

  if (is.null(l.window)) {
    l.window <- nextodd(n.p)
  } else {
    l.window <- wincheck(l.window)
  }

  if (is.null(sub.labels)) {
    sub.labels <- paste("subseries", 1:n.p)
  } else {
    if (length(sub.labels) != n.p) {
      stop("sub.labels must be of length n.p")
    }
    if (length(unique(sub.labels)) != n.p) stop("sub.labels must be unique")
  }
  tmp <- c(sub.start:n.p, rep(1:n.p, ceiling(n / n.p)))[1:n]
  sub.labels <- factor(tmp, labels = sub.labels)

  periodic <- FALSE
  if (is.character(s.window)) {
    if (is.na(pmatch(s.window, "periodic"))) {
      stop("unknown string value for s.window")
    } else {
      periodic <- TRUE
      s.window <- 10 * n + 1
      s.degree <- 0
      s.jump <- ceiling(s.window / 10)
    }
  } else {
    s.window <- wincheck(s.window)
  }

  degcheck(s.degree)
  degcheck(t.degree)
  degcheck(l.degree)

  if (is.null(t.window)) {
    # t.window <- nextodd(ceiling(1.5 * n.p/(1 - 1.5 / s.window)))
    t.window <- get.t.window(t.degree, s.degree, s.window, n.p, critfreq)
  } else {
    t.window <- wincheck(t.window)
  }

  if (is.null(s.jump) || length(s.jump) == 0) {
    s.jump <- ceiling(s.window / 10)
  }
  if (is.null(t.jump) || length(t.jump) == 0) {
    t.jump <- ceiling(t.window / 10)
  }
  if (is.null(l.jump) || length(l.jump) == 0) {
    l.jump <- ceiling(l.window / 10)
  }

  # cat(s.degree, " ", t.degree, " ", l.degree, "\n")
  # cat(s.window, " ", t.window, " ", l.window, "\n")

  # Trend vector - initialize to 0 or NA, depending on what's in Y
  trend <- 0

  # start and end indices for after adding in extra n.p before and after
  st <- n.p + 1
  nd <- n + n.p

  # cycleSubIndices will keep track of what part of the
  # seasonal each observation belongs to
  cycleSubIndices <- rep(c(1:n.p), ceiling(n / n.p))[1:n]

  if (any(by(Y, list(cycleSubIndices), function(x) all(is.na(x))))) {
    stop("There is at least one subseries for which all values are missing.")
  }

  C <- rep(NA, n + 2 * n.p)

  dtls <- NULL

  w <- rep(1, n)

  for (o_iter in 1:outer) {
    for (iter in 1:inner) {
      # step 1: detrending...
      Y.detrended <- Y - trend

      csLength <- ceiling(n / n.p) + 2

      # step 2: smoothing of cycle-subseries
      for (i in 1:n.p) {
        cycle_sub <- Y.detrended[cycleSubIndices == i]
        # sub_weights <- w[cycleSubIndices == i]
        cycle_sub_length <- length(cycle_sub)

        cs1 <- head(cycleSubIndices, n.p)
        cs2 <- tail(cycleSubIndices, n.p)

        # not_enough_dat <- length(cycle_sub[!is.na(cycle_sub)]) < s.window / 2
        # if(not_enough_dat || periodic) {
        if (periodic) {
          C[c(cs1, cycleSubIndices, cs2) == i] <- rep(
            weighted.mean(cycle_sub, w = w[cycleSubIndices == i], na.rm = TRUE),
            cycle_sub_length + 2
          )
        } else {
          cs.ev <- seq(1, cycle_sub_length, by = s.jump)
          if (tail(cs.ev, 1) != cycle_sub_length) {
            cs.ev <- c(cs.ev, cycle_sub_length)
          }
          cs.ev <- c(0, cs.ev, cycle_sub_length + 1)
          tmps <- .loess_stlplus(
            y = cycle_sub,
            span = s.window,
            degree = s.degree,
            m = cs.ev,
            weights = w[cycleSubIndices == i],
            blend = s.blend,
            jump = s.jump,
            at = c(0:(cycle_sub_length + 1))
          )
          C[c(cs1, cycleSubIndices, cs2) == i] <- tmps
          # approx(x = cs.ev, y = tmps, xout = c(0:(cycle_sub_length + 1)))$y
        }
      }

      # Step 3: Low-pass filtering of collection of all the cycle-subseries
      # moving averages
      ma3 <- c_ma(C, n.p)

      l.ev <- seq(1, n, by = l.jump)
      if (tail(l.ev, 1) != n) {
        l.ev <- c(l.ev, n)
      }
      L <- .loess_stlplus(
        y = ma3,
        span = l.window,
        degree = l.degree,
        m = l.ev,
        weights = w,
        y_idx = y_idx,
        noNA = noNA,
        blend = l.blend,
        jump = l.jump,
        at = c(1:n)
      )

      # L <- predict(loess(ma3 ~ c(1:n), degree = l.degree,
      #   span = l.window / n, family = family), newdata = c(1:n))

      # Step 4: Detrend smoothed cycle-subseries
      seasonal <- C[st:nd] - L

      # Step 5: Deseasonalize
      D <- Y - seasonal

      # Step 6: Trend Smoothing

      t.ev <- seq(1, n, by = t.jump)
      if (tail(t.ev, 1) != n) {
        t.ev <- c(t.ev, n)
      }
      trend <- .loess_stlplus(
        y = D,
        span = t.window,
        degree = t.degree,
        m = t.ev,
        weights = w,
        y_idx = y_idx,
        noNA = noNA,
        blend = t.blend,
        jump = t.jump,
        at = c(1:n)
      )
      # if(blend$t) {
      #   # TODO: validate blend parameters
      #   trend0 <- .loess_stlplus(y = D, span = t.window, degree = t.degree,
      #     m = t.ev, weights = w, y_idx = y_idx, noNA = noNA)
      # }

      # trend <- predict(loess(D ~ c(1:length(D)), degree = t.degree,
      #   span = t.window / length(D), family = family), newdata = c(1:length(D)))

      if (details) {
        dtls <- c(
          dtls,
          list(list(
            trend = trend,
            seasonal = seasonal,
            C = C,
            D = D,
            L = L,
            Y.detrended = Y.detrended,
            weights = w
          ))
        )
      }
    }

    if (outer > 1) {
      mid1 <- floor(n / 2 + 1)
      mid2 <- n - mid1 + 1
      R <- Y - seasonal - trend
      R.abs <- abs(R)
      h <- 3 * sum(sort(R.abs)[mid1:mid2])
      h9 <- 0.999 * h
      h1 <- 0.001 * h
      w <- (1 - (R.abs / h)^2)^2
      w[R.abs <= h1] <- 1
      w[R.abs >= h9] <- 0
      w[w == 0] <- zero.weight
      w[is.na(w)] <- 1
    }
  }

  ## post-seasonal smoothing, if specified
  fc <- NULL
  fc.number <- 0
  fc.res <- NULL
  if (!is.null(fc.window)) {
    fc.number <- length(fc.window)
    fc.window <- wincheck(fc.window)
    if (is.null(fc.degree)) {
      fc.degree <- 1
    }
    if (length(fc.degree) < fc.number) {
      fc.degree <- c(
        fc.degree,
        rep(fc.degree[length(fc.degree)], fc.number - length(fc.degree))
      )
    }
    fc.cumulative <- rep(0, n) # keep sum of all previous fc smoothings

    degcheck(fc.degree)

    if (is.null(fc.name)) {
      fc.name <- paste("fc.", fc.window, sep = "")
    }

    if (is.null(fc.jump)) {
      fc.jump <- ceiling(fc.window / 10)
    }

    if (is.null(fc.blend)) {
      fc.blend <- rep(t.blend, fc.number)
    }

    if (length(fc.blend) < fc.number) {
      fc.blend <- c(fc.blend, rep(0, fc.number - length(fc.blend)))
    }

    if (length(fc.jump) < fc.number) {
      fc.jump <- c(
        fc.jump,
        rep(fc.jump[length(fc.jump)], fc.number - length(fc.jump))
      )
    }

    fc.res <- data.frame(matrix(nrow = n, ncol = fc.number, data = 0))

    for (ii in 1:fc.number) {
      fc.ev <- seq(1, n, by = fc.jump[ii])
      if (tail(fc.ev, 1) != n) {
        fc.ev <- c(fc.ev, n)
      }
      tmp <- .loess_stlplus(
        y = Y - seasonal - fc.cumulative,
        span = fc.window[ii],
        degree = fc.degree[ii],
        m = fc.ev,
        weights = w,
        y_idx = y_idx,
        noNA = noNA,
        blend = fc.blend[ii],
        jump = fc.jump[ii],
        at = c(1:n)
      )

      fc.cumulative <- fc.cumulative + tmp
      fc.res[, ii] <- tmp
    }

    if (any(is.null(fc.name)) || length(fc.name) < fc.number) {
      fc.name <- paste("trend", c(1:fc.number))
    }

    names(fc.res) <- fc.name
    fc.res$remainder <- Y - seasonal - apply(fc.res, 1, sum)

    fc <- data.frame(
      fc.window = fc.window,
      fc.degree = fc.degree,
      fc.name = fc.name,
      fc.jump = fc.jump,
      fc.blend = fc.blend
    )
  }

  # compute remainder
  R <- Y - seasonal - trend

  dat <- data.frame(
    raw = Y,
    seasonal = seasonal,
    trend = trend,
    remainder = R,
    weights = w,
    sub.labels = sub.labels
  )

  pars <- list(
    deg = data.frame(
      s.degree = s.degree,
      t.degree = t.degree,
      l.degree = l.degree
    ),
    win = data.frame(
      s.window = s.window,
      t.window = t.window,
      l.window = l.window
    ),
    blend = data.frame(s.blend = s.blend, t.blend = t.blend, l.blend = l.blend),
    jump = data.frame(s.jump = s.jump, t.jump = t.jump, l.jump = l.jump),
    fc.number = fc.number,
    fc = fc,
    n.p = n.p,
    inner = inner,
    outer = outer,
    periodic = periodic
  )

  res <- list(
    data = dat,
    fc = fc.res,
    pars = pars,
    time = t,
    n = nrow(dat),
    details = dtls
  )

  class(res) <- "stlplus"

  res
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

c_interp <- function(m, fits, slopes, at) {
    .Call(`_stlplus_c_interp`, m, fits, slopes, at)
}

c_loess <- function(xx, yy, degree, span, ww, m, l_idx, max_dist) {
    .Call(`_stlplus_c_loess`, xx, yy, degree, span, ww, m, l_idx, max_dist)
}

c_ma <- function(x, n_p) {
    .Call(`_stlplus_c_ma`, x, n_p)
}

#' @importFrom yaImpute ann
#' @useDynLib stlplus, .registration = TRUE
.loess_stlplus <- function(
  x = NULL,
  y,
  span,
  degree,
  weights = NULL,
  m = c(seq_along(y)),
  y_idx = !is.na(y),
  noNA = all(y_idx),
  blend = 0,
  jump = ceiling(span / 10),
  at = c(seq_along(y))
) {
  nextodd <- function(x) {
    x <- round(x)
    x2 <- ifelse(x %% 2 == 0, x + 1, x)
    as.integer(x2)
  }

  n <- length(y[y_idx])
  if (is.null(x)) {
    x <- c(seq_along(y))
  }

  if (is.null(weights)) {
    weights <- rep(1, length(y))
  }
  n_m <- length(m)

  if ((span %% 2) == 0) {
    span <- span + 1
    warning(paste(
      "Span must be odd! Changed span from ",
      span - 1,
      " to ",
      span,
      sep = ""
    ))
  }

  s2 <- (span + 1) / 2
  # set up indices in R - easier
  if (noNA) {
    if ((diff(range(x))) < span) {
      l_idx <- rep(1, n_m)
      r_idx <- rep(n, n_m)
    } else {
      l_idx <- c(
        rep(1, length(m[m < s2])),
        m[m >= s2 & m <= n - s2] - s2 + 1,
        rep(n - span + 1, length(m[m > n - s2]))
      )
      r_idx <- l_idx + span - 1
    }
    aa <- abs(m - x[l_idx])
    bb <- abs(x[r_idx] - m)
    max_dist <- ifelse(aa > bb, aa, bb)
    # max_dist <- apply(cbind(abs(m - x[l_idx]), abs(x[r_idx] - m)), 1, max)
  } else {
    span3 <- min(span, n)
    x2 <- x[y_idx]

    # another approach
    a <- yaImpute::ann(
      ref = as.matrix(x2),
      target = as.matrix(m),
      tree.type = "kd",
      k = span3,
      eps = 0,
      verbose = FALSE
    )$knnIndexDist[, 1:span3, drop = FALSE]

    l_idx <- apply(a, 1, min)
    r_idx <- apply(a, 1, max)

    max_dist <- apply(cbind(abs(m - x2[l_idx]), abs(x2[r_idx] - m)), 1, max)
  }
  if (span >= n) {
    # max_dist <- max_dist * (span / n)
    max_dist <- max_dist + (span - n) / 2
  }

  out <- c_loess(
    x[y_idx],
    y[y_idx],
    degree,
    span,
    weights[y_idx],
    m,
    l_idx - 1,
    as.double(max_dist)
  )

  res1 <- out$result
  # do interpolation
  if (jump > 1) {
    res1 <- .interp(m, out$result, out$slope, at)
  }
  # res1 <- approx(x = m, y = out$result, xout = at)$y

  if (blend > 0 && blend <= 1 && degree >= 1) {
    if (degree == 2) {
      sp0 <- nextodd((span + 1) / 2)
    }
    if (degree == 1) {
      sp0 <- span
    }

    n.b <- as.integer(span / 2)

    blend <- 1 - blend # originally programmed backwards - easier to fix this way
    # indices for left and right blending points
    # take into account if n_m is too small
    mid <- median(m)
    bl_idx <- m <= n.b + jump & m < mid
    br_idx <- m >= n - n.b - jump + 1 & m >= mid
    left <- m[bl_idx]
    right <- m[br_idx]
    bl_idx_interp <- at <= max(left)
    br_idx_interp <- at >= min(right)
    left_interp <- at[bl_idx_interp]
    right_interp <- at[br_idx_interp]
    # left_interp <- at[bl_idx_interp]
    # right_interp <- at[br_idx_interp]
    l_idx2 <- l_idx[bl_idx | br_idx]
    # r_idx2 <- r_idx[bl_idx | br_idx]
    max_dist2 <- max_dist[bl_idx | br_idx]

    m2 <- c(left, right)
    # n_m2 <- length(m2)

    # speed this up later by only getting the loess smooth at the tails.
    # right now, a lot of unnecessary calculation is done at the interior
    # where blending doesn't matter

    tmp <- c_loess(
      x[y_idx],
      y[y_idx],
      0,
      sp0,
      weights[y_idx],
      m2,
      l_idx2 - 1,
      max_dist2
    )

    if (jump > 1) {
      res2_left <- .interp(
        left,
        head(tmp$result, length(left)),
        head(tmp$slope, length(left)),
        left_interp
      )
      res2_right <- .interp(
        right,
        tail(tmp$result, length(right)),
        tail(tmp$slope, length(right)),
        right_interp
      )
    } else {
      res2_left <- head(tmp$result, length(left))
      res2_right <- tail(tmp$result, length(right))
    }
    # res2 <- approx(x = m, y = tmp$result, xout = at)$y

    p.left <- ((1 - blend) / (n.b - 1)) * (left_interp - 1) + blend
    p.right <- ((blend - 1) / (n.b - 1)) * (right_interp - (n - n.b + 1)) + 1
    p.left[p.left < blend] <- blend
    p.left[p.left > 1] <- 1
    p.right[p.right < blend] <- blend
    p.right[p.right > 1] <- 1

    res1[bl_idx_interp] <- res1[bl_idx_interp] *
      p.left +
      res2_left * (1 - p.left)
    res1[br_idx_interp] <- res1[br_idx_interp] *
      p.right +
      res2_right * (1 - p.right)

    # xxx <- x[y_idx]
    # yyy <- y[y_idx]
    # tmp2 <- predict(loess(yyy ~ xxx, deg = 0, span=(sp0 + 0.00000001) / length(yyy), control = loess.control(surface = "direct")), newdata = m2)
    # tmp3 <- predict(loess(yyy ~ xxx, deg = degree, span=(span + 0.00000001) / length(yyy), control = loess.control(surface = "direct")), newdata = m)
  }

  res1
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{plot_seasonal}
\alias{plot_seasonal}
\title{Seasonal Diagnostic Plot for an stlplus Object}
\usage{
plot_seasonal(
  x,
  col = c("darkgray", "black"),
  lwd = 2,
  xlab = "Time",
  ylab = "Seasonal + Remainder\\n(centered by seasonal mean)",
  ...
)
}
\arguments{
\item{x}{object of class \code{"stlplus"}.}

\item{col, lwd, xlab, ylab, \ldots}{parameters to be passed to \code{xyplot()}.}
}
\value{
object of class \code{"trellis"}.
}
\description{
Plots each cycle-subseries of the detrended data (or equivalently,
seasonal plus remainder), centered by the mean seasonal component for each
subseries. The fitted seasonal component is plotted as a line through the
points.
}
\details{
Helps decide how much of the variation in the data other than the trend should go into the seasonal component, and how much in the remainder.
}
\references{
R. B. Cleveland, W. S. Cleveland, J. E.  McRae, and I. Terpenning (1990) STL:  A  Seasonal-Trend  Decomposition  Procedure Based on Loess. \emph{Journal of Official Statistics}, \bold{6}, 3--73.
}
\seealso{
\code{\link{stlplus}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{plot_cycle}
\alias{plot_cycle}
\title{Cycle-Subseries Plot for an stlplus Object}
\usage{
plot_cycle(
  x,
  layout = c(x$pars$n.p, 1),
  col = "#0080ff",
  xlab = "Time",
  ylab = "Seasonal",
  panel = function(x, y, ...) {
     lattice::panel.segments(x, rep(.midmean(y),
    length(x)), x, y, col = col)
 },
  ...
)
}
\arguments{
\item{x}{object of class \code{"stlplus"}.}

\item{layout, col, xlab, ylab, panel, \ldots}{parameters to be passed to \code{xyplot()}.}
}
\value{
object of class \code{"trellis"}.
}
\description{
Plots the seasonal component by cycle-subseries, with lines emanating
from the midmean of the values within each cycle-subseries.
}
\references{
R. B. Cleveland, W. S. Cleveland, J. E.  McRae, and I. Terpenning (1990) STL:  A  Seasonal-Trend  Decomposition  Procedure Based on Loess. \emph{Journal of Official Statistics}, \bold{6}, 3--73.
}
\seealso{
\code{\link{stlplus}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{plot_rembycycle}
\alias{plot_rembycycle}
\title{Plot of Remainder Component by Cycle-Subseries}
\usage{
plot_rembycycle(
  x,
  col = "darkgray",
  locol = "black",
  lolwd = 2,
  xlab = "Time",
  ylab = "Remainder",
  ...
)
}
\arguments{
\item{x}{object of class \code{"stlplus"}.}

\item{col, locol, lolwd, xlab, ylab, \ldots}{parameters to be passed to \code{xyplot()}. \code{locol} and \code{lolwd} are the line color and width for the loess line.}
}
\value{
object of class \code{"trellis"}.
}
\description{
Plots the remainder component by cycle-subseries with a loess line.
}
\references{
R. B. Cleveland, W. S. Cleveland, J. E.  McRae, and I. Terpenning (1990) STL:  A  Seasonal-Trend  Decomposition  Procedure Based on Loess. \emph{Journal of Official Statistics}, \bold{6}, 3--73.
}
\seealso{
\code{\link{stlplus}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{plot.stlplus}
\alias{plot.stlplus}
\title{Lattice plot of the raw, seasonal, trend, and remainder components}
\usage{
\method{plot}{stlplus}(
  x,
  scales = list(y = list(relation = "sliced")),
  type = "l",
  as.table = TRUE,
  strip = FALSE,
  strip.left = TRUE,
  between = list(y = 0.5),
  layout = NULL,
  ...
)
}
\arguments{
\item{x}{object of class \code{"stlplus"}.}

\item{scales, type, as.table, strip, strip.left, between, layout, \ldots}{parameters to be passed to xyplot.}
}
\value{
object of class \code{"trellis"}.
}
\description{
Lattice plot of the raw, seasonal, trend, and remainder components.  If post-trend smoothing was done, these components will be plotted instead of the trend component.
}
\references{
R. B. Cleveland, W. S. Cleveland, J. E.  McRae, and I. Terpenning (1990) STL:  A  Seasonal-Trend  Decomposition  Procedure Based on Loess. \emph{Journal of Official Statistics}, \bold{6}, 3--73.
}
\seealso{
\code{\link{stlplus}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accessors.R
\name{seasonal}
\alias{seasonal}
\alias{trend}
\alias{remainder}
\alias{getraw}
\alias{remainder.stlplus}
\alias{fitted.stlplus}
\alias{predict.stlplus}
\alias{seasonal.stlplus}
\alias{trend.stlplus}
\alias{fc}
\alias{time.stlplus}
\alias{remainder.stl}
\alias{seasonal.stl}
\alias{trend.stl}
\alias{time.stl}
\alias{predict.stl}
\alias{fitted.stl}
\title{Accessor functions for elements of an stl and stlplus object}
\usage{
seasonal(object)

trend(object)

remainder(object)

getraw(object)

\method{remainder}{stlplus}(object)

\method{fitted}{stlplus}(object, ...)

\method{predict}{stlplus}(object, ...)

\method{seasonal}{stlplus}(object)

\method{trend}{stlplus}(object)

fc(object, fcnum = 1)

\method{time}{stlplus}(x, ...)

\method{remainder}{stl}(object)

\method{seasonal}{stl}(object)

\method{trend}{stl}(object)

\method{time}{stl}(x, ...)

\method{predict}{stl}(object, ...)

\method{fitted}{stl}(object, ...)
}
\arguments{
\item{fcnum}{number of post-trend smoothing frequency component.}

\item{x, object}{object of class \code{"stl"} or \code{"stlplus"}.}

\item{\ldots}{additional parameters}
}
\value{
Returns a vector of either the \code{getraw} time series, the \code{seasonal}, \code{trend}, or \code{remainder} components, or the \code{time} values of the time series.  If \code{time}s are requested but were not supplied in the initial \code{stlplus} call, the \code{1:n} vector is returned, where \code{n} is the number of data points.  The \code{fitted} method returns the sum of the seasonal and trend.
}
\description{
Retrieves the raw, seasonal, trend, remainder, or time components from an stlplus object.  The methods \code{seasonal.stl}, ... also exist as a convenience for extracting components from R's \code{stl()}.
}
\note{
The \code{fitted} and \code{predict} methods are equivalent.  For objects of class \code{"stlplus"}, these functions return the sum of all components but the remainder, including post-trend smoothing components.  Note also that the \code{trend} method for objects of class \code{"stlplus"} only returns the trend component from the STL iterations, even when post-trend smoothing is done.
}
\examples{
co2.stl <- stlplus(co2, t = as.vector(stats::time(co2)), n.p=12, l.window=13,
t.window=19, s.window=35, s.degree=1, sub.labels = substr(month.name, 1, 3))

plot(seasonal(co2.stl))
}
\references{
R. B. Cleveland, W. S. Cleveland, J.E.  McRae, and I. Terpenning (1990) STL:  A  Seasonal-Trend  Decomposition  Procedure Based on Loess. \emph{Journal of Official Statistics}, \bold{6}, 3--73.
}
\seealso{
\code{\link{stlplus}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stlplus.R
\name{stlplus}
\alias{stlplus}
\alias{stlplus.ts}
\alias{stlplus.zoo}
\title{Seasonal Decomposition of Time Series by Loess}
\usage{
stlplus(
  x,
  t = NULL,
  n.p,
  s.window,
  s.degree = 1,
  t.window = NULL,
  t.degree = 1,
  fc.window = NULL,
  fc.degree = NULL,
  fc.name = NULL,
  l.window = NULL,
  l.degree = t.degree,
  s.jump = ceiling(s.window/10),
  t.jump = ceiling(t.window/10),
  l.jump = ceiling(l.window/10),
  fc.jump = NULL,
  critfreq = 0.05,
  s.blend = 0,
  t.blend = 0,
  l.blend = t.blend,
  fc.blend = NULL,
  inner = 2,
  outer = 1,
  sub.labels = NULL,
  sub.start = 1,
  zero.weight = 0.000001,
  details = FALSE,
  ...
)

\method{stlplus}{ts}(
  x,
  t = as.numeric(stats::time(x)),
  n.p = frequency(x),
  s.window,
  s.degree = 1,
  t.window = NULL,
  t.degree = 1,
  fc.window = NULL,
  fc.degree = NULL,
  fc.name = NULL,
  l.window = NULL,
  l.degree = t.degree,
  s.jump = ceiling(s.window/10),
  t.jump = ceiling(t.window/10),
  l.jump = ceiling(l.window/10),
  fc.jump = NULL,
  critfreq = 0.05,
  s.blend = 0,
  t.blend = 0,
  l.blend = t.blend,
  fc.blend = NULL,
  inner = 2,
  outer = 1,
  sub.labels = NULL,
  sub.start = 1,
  zero.weight = 0.000001,
  details = FALSE,
  ...
)

\method{stlplus}{zoo}(...)
}
\arguments{
\item{x}{vector of time series values, in order of time.  If \code{x} is a time series object, then \code{t} and \code{n.p} do not need to be specified, although they still can be.}

\item{t}{times at which the time series values were observed.  Not required.}

\item{n.p}{periodicity of the seasonal component.  In R's \code{stl} function, this is the frequency of the time series.}

\item{s.window}{either the character string \code{"periodic"} or the span (in lags) of the loess window for seasonal extraction, which should be odd.  This has no default.}

\item{s.degree}{degree of locally-fitted polynomial in seasonal extraction.  Should be 0, 1, or 2.}

\item{t.window}{the span (in lags) of the loess window for trend extraction, which should be odd.  If \code{NULL}, the default, \code{nextodd(ceiling((1.5*period) / (1-(1.5/s.window))))}, is taken.}

\item{t.degree}{degree of locally-fitted polynomial in trend extraction.  Should be 0, 1, or 2.}

\item{fc.window}{vector of lengths of windows for loess smoothings for other trend frequency components after the original STL decomposition has been obtained.  The smoothing is applied to the data with the STL seasonal component removed.  A frequency component is computed by a loess fit with the window length equal to the first element of fc.window, the component is removed, another component is computed with the window length equal to the second element of fc.window, and so forth. In most cases, the values of the argument should be decreasing, that is, the frequency bands of the fitted components should increase. The robustness weights from original STL are used as weights in the loess fitting if specified.}

\item{fc.degree}{vector of degrees of locally-fitted polynomial in the loess smoothings for the frequency components specified in fc.window. Values of 0, 1 and 2 are allowed. If the length of fc.degree is less than that of fc.window, the former is expanded to the length of the latter using rep; thus, giving the value 1 specifies a degree of 1 for all components.}

\item{fc.name}{vector of names of the post-trend smoothing operations specified by \code{fc.window} and \code{fc.degree} (optional).}

\item{l.window}{the span (in lags) of the loess window of the low-pass filter used for each subseries.  Defaults to the smallest odd integer greater than or equal to \code{n.p} which is recommended since it prevents competition between the trend and seasonal components.  If not an odd integer its given value is increased to the next odd one.}

\item{l.degree}{degree of locally-fitted polynomial for the subseries low-pass filter.  Should be 0, 1, or 2.}

\item{s.jump, t.jump, l.jump, fc.jump}{integers at least one to increase speed of the respective smoother.  Linear interpolation happens between every \code{*.jump}th value.}

\item{critfreq}{the critical frequency to use for automatic calculation of smoothing windows for the trend and high-pass filter.}

\item{s.blend, t.blend, l.blend, fc.blend}{vectors of proportion of blending to degree 0 polynomials at the endpoints of the series.}

\item{inner}{integer; the number of \sQuote{inner} (backfitting) iterations; usually very few (2) iterations suffice.}

\item{outer}{integer; the number of \sQuote{outer} robustness iterations.  Default is 0, but Recommended if outliers are present.}

\item{sub.labels}{optional vector of length n.p that contains the labels of the subseries in their natural order (such as month name, day of week, etc.), used for strip labels when plotting.  All entries must be unique.}

\item{sub.start}{which element of sub.labels does the series begin with.  See details.}

\item{zero.weight}{value to use as zero for zero weighting}

\item{details}{if \code{TRUE}, returns a list of the results of all the intermediate iterations.}

\item{\ldots}{additional parameters}
}
\value{
returns an object of class \code{"stlplus"}, containing
\item{data}{data frame containing all of the components: \code{raw}, \code{seasonal}, \code{trend}, \code{remainder}, \code{weights}.}
\item{pars}{list of parameters used in the procedure.}
\item{fc.number}{number of post-trend frequency components fitted.}
\item{fc}{data frame of the post-trend frequency components.}
\item{time}{vector of time values corresponding to the raw values, if specified.}
\item{n}{the number of observations.}
\item{sub.labels}{the cycle-subseries labels.}
}
\description{
Decompose a time series into seasonal, trend and irregular components using \code{loess}, acronym STL.  A new implementation of STL.  Allows for NA values, local quadratic smoothing,  post-trend smoothing, and endpoint blending.  The usage is very similar to that of R's built-in \code{stl()}.
}
\details{
The seasonal component is found by \emph{loess} smoothing the seasonal sub-series (the series of all January values, \ldots); if \code{s.window = "periodic"} smoothing is effectively replaced by taking the mean. The seasonal values are removed, and the remainder smoothed to find the trend. The overall level is removed from the seasonal component and added to the trend component. This process is iterated a few times.  The \code{remainder} component is the residuals from the seasonal plus trend fit.

Cycle-subseries labels are useful for plotting and can be specified through the sub.labels argument.  Here is an example for how the sub.labels and sub.start parameters might be set for one situation.  Suppose we have a daily series with n.p=7 (fitting a day-of-week component).  Here, sub.labels could be set to c("Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat").  Now, if the series starts with a Wednesday value, then one would specify sub.labels=4, since Wednesday is the fourth element of sub.labels.  This ensures that the labels in the plots to start the plotting with Sunday cycle-subseries instead of Wednesday.
}
\note{
This is a complete re-implementation of the STL algorithm, with the loess part in C and the rest in R.  Moving a lot of the code to R makes it easier to experiment with the method at a very minimal speed cost.  Recoding in C instead of using R's built-in loess results in better performance, especially for larger series.
}
\examples{
co2_stl <- stlplus(
  co2,
  t = as.vector(time(co2)),
  n.p = 12,
  l.window = 13,
  t.window = 19,
  s.window = 35,
  s.degree = 1,
  sub.labels = substr(month.name, 1, 3)
)

plot(co2_stl, ylab = "CO2 Concentration (ppm)", xlab = "Time (years)")
plot_seasonal(co2_stl)
plot_trend(co2_stl)
plot_cycle(co2_stl)
plot_rembycycle(co2_stl)

# post-trend smoothing

co2_stl_pt <- stlplus(
  co2,
  t = as.vector(time(co2)),
  n.p = 12,
  l.window = 13,
  t.window = 19,
  s.window = 35,
  s.degree = 1,
  sub.labels = substr(month.name, 1, 3),
  fc.degree = c(1, 2),
  fc.window = c(201, 35),
  fc.name = c("long-term", "so. osc.")
)

plot(
  co2_stl_pt,
  scales = list(y = list(relation = "free")),
  ylab = "CO2 Concentration (ppm)",
  xlab = "Time (years)",
  aspect = 0.25,
  type = c("l", "g")
)

# with NAs

y <- co2
y[201:224] <- NA

y_stl <- stlplus(
  y,
  l.window = 13,
  t.window = 19,
  s.window = 35,
  s.degree = 1,
  sub.labels = substr(month.name, 1, 3)
)

plot(
  y_stl,
  ylab = "CO2 Concentration (ppm)",
  xlab = "Time (years)",
  type = c("l", "g")
)
plot_seasonal(y_stl)
plot_trend(y_stl)
plot_cycle(y_stl)
plot_rembycycle(y_stl)

# if you don't want to use a time series object:
y_stl <- stlplus(
  y,
  t = as.vector(time(y)),
  n.p = 12,
  l.window = 13,
  t.window = 19,
  s.window = 35,
  s.degree = 1,
  sub.labels = substr(month.name, 1, 3)
)

# with an outlier
y2 <- co2
y2[200] <- 300

y2_stl <- stlplus(
  y2,
  t = as.vector(time(y2)),
  n.p = 12,
  l.window = 13,
  t.window = 19,
  s.window = 35,
  s.degree = 1,
  sub.labels = substr(month.name, 1, 3),
  outer = 10
)

plot(y2_stl, ylab = "CO2 Concentration (ppm)", xlab = "Time (years)")
plot_seasonal(y2_stl)
plot_trend(y2_stl)
plot_cycle(y2_stl)
plot_rembycycle(y2_stl)

# compare to R's stl

x1 <- stlplus(
  co2,
  t = as.vector(time(co2)),
  n.p = 12,
  l.window = 13,
  t.window = 19,
  s.window = 11,
  s.degree = 1,
  sub.labels = substr(month.name, 1, 3)
)

x2 <- stl(co2, l.window = 13, t.window = 19, s.window = 11, s.degree = 1)

# will be different due to interpolation differences
plot(seasonal(x1) - seasonal(x2))

# but not if all jump parameters are 1
x1 <- stlplus(
  co2,
  t = as.vector(time(co2)),
  n.p = 12,
  l.window = 13,
  t.window = 19,
  s.window = 11,
  s.degree = 1,
  sub.labels = substr(month.name, 1, 3),
  s.jump = 1,
  t.jump = 1,
  l.jump = 1
)

x2 <- stl(
  co2,
  l.window = 13,
  t.window = 19,
  s.window = 11,
  s.degree = 1,
  s.jump = 1,
  t.jump = 1,
  l.jump = 1
)

plot(seasonal(x1) - seasonal(x2))
}
\references{
R. B. Cleveland, W. S. Cleveland, J. E.  McRae, and I. Terpenning (1990) STL:  A  Seasonal-Trend  Decomposition  Procedure Based on Loess. \emph{Journal of Official Statistics}, \bold{6}, 3--73.
}
\seealso{
\code{\link{plot.stlplus}} for plotting the components, \code{\link{getraw}}, \code{\link{seasonal}}, \code{\link{trend}}, \code{\link{remainder}} for accessing the components.
}
\author{
Ryan Hafen
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{plot_trend}
\alias{plot_trend}
\title{Trend Diagnostic Plot for an stlplus Object}
\usage{
plot_trend(
  x,
  xlab = "Time",
  ylab = "Trend",
  span = 0.3,
  type = c("p", "l"),
  scales = list(y = list(relation = "free")),
  lwd = c(1, 1),
  col = c("darkgray", "black", "darkgray"),
  layout = c(1, 2),
  between = list(y = 0.5),
  strip = FALSE,
  strip.left = TRUE,
  as.table = TRUE,
  ...
)
}
\arguments{
\item{x}{object of class \code{"stlplus"}.}

\item{xlab, ylab, span, type, scales, lwd, col, layout}{parameters to be passed to
xyplot.}

\item{between, strip, strip.left, as.table, \ldots}{parameters to be passed to
xyplot.}
}
\value{
object of class \code{"trellis"}.
}
\description{
Plots the trend+remainder with the trend component overlaid, and the
remainder component in a separate panel.
}
\references{
R. B. Cleveland, W. S. Cleveland, J. E.  McRae, and I. Terpenning (1990) STL:  A  Seasonal-Trend  Decomposition  Procedure Based on Loess. \emph{Journal of Official Statistics}, \bold{6}, 3--73.
}
\seealso{
\code{\link{stlplus}}
}
